--[[
	Copyright (C) Udorn (Blackhand)
--]]

qmstr = qmstr or {}

qmstr.Config = qmstr.QuestMaster:NewModule("Config", "AceEvent-3.0")
local L = qmstr.Locale.GetInstance()
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")

local log = vendor.Debug:new("Config")

local self = qmstr.Config

-- quest filter types
qmstr.Config.FILTER_OFF = 1
qmstr.Config.FILTER_ZONE = 2
qmstr.Config.FILTER_WATCH = 3
qmstr.Config.FILTER_WATCH_OR_ZONE = 4

qmstr.Config.DEFAULT_CONFIG = {
	profile = {
		windows = {},
		ignoredQuests = {},
		ignoredAchievements = {},
		locked = true,
		headerPaddingTop = 6,
		headerPaddingLeft = 30,
		headerColor = {ORANGE_FONT_COLOR.r, ORANGE_FONT_COLOR.g, ORANGE_FONT_COLOR.b, 1},
		entryPaddingTop = 4,
		entryPaddingLeft = 5,
		colorizeQuestDifficulty = true,
		entryColor = {1, 1, 1, 1},
		objectivePaddingTop = 3,
		objectivePaddingLeft = 10,
		objectiveColor = {GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b, 1},
		colorizeQuestObjectives = true,
		maxAchievementTextLength = 512,
		maxAchievementCriteriaLength = 40,
		questCompleteSound = "SOUND-UI_AutoQuestComplete",
		filterMode = qmstr.Config.FILTER_WATCH_OR_ZONE,
		noFilterIfEmpty = true,
		showHeaders = true,
	}
}

local function _NextOrder()
	local order = self.nextOrder or 1
	self.nextOrder = order + 1
	return order
end

local function _InitOpts(self)
	self.opts = {
    	type = "group",
    	childGroups = "tree",
      	args = {
      		lock = {
				order = _NextOrder(),
				type = "toggle",
				name = L["Lock window"],
				desc = L["Locks the QuestMaster window for movement and resizing."],
				get = function() return self.profile.locked end,
				set = function(info, value) self.profile.locked = value; self:Update() end,
			},
			misc = {
        		type = "group",
        		name = L["Miscellaneous"],
        		order = _NextOrder(),
      			args = {
   					newbiehelp = {
    					type = "description",
    					name = L["NEWBIE_HELP"].."\n\n\n",
    					order = _NextOrder()	
    				},
    				filterMode = {
    					type = "select",
    					name = L["Filter mode"],
    					desc = L["Filters the list of quests shown in the QuestMaster window."],
    					order = _NextOrder(),
    					values = {
    						[qmstr.Config.FILTER_OFF] = L["Off"],
    						[qmstr.Config.FILTER_ZONE] = L["Current zone"],
    						[qmstr.Config.FILTER_WATCH] = L["Quest watch"],
    						[qmstr.Config.FILTER_WATCH_OR_ZONE] = L["Current zone and quest watch"]
    					},
    					set = function(info, value) self.profile.filterMode = value; self:Update() end,
    					get = function() return self.profile.filterMode end,
    				},
    				noFilterIfEmpty = {
	    				type = "toggle",
	    				name = L["No filter if empty"],
	    				desc = L["Turns off the filter, if it would result in an empty quest list."],
	    				order = _NextOrder(),
	    				set = function(info, value) self.profile.noFilterIfEmpty = value; self:Update() end,
	    				get = function() return self.profile.noFilterIfEmpty end,
    				},
    				showHeaders = {
    					type = "toggle",
    					name = L["Show headers"],
    					desc = L["Show the quest zones and the achievement header."],
    					order = _NextOrder(),
    					set = function(info, value) self.profile.showHeaders = value; self:Update() end,
    					get = function() return self.profile.showHeaders end,
    				},
      				questCompleteSound = {
      					type = "select",
      					order = _NextOrder(),
      					name = L["Quest complete sound"],
      					desc = L["Sound played when completing a quest."],
      					values = {
      						[""] = L["-None-"],
      						["SOUND-QUESTCOMPLETED"] = L["Quest completed"],
      						["SOUND-UI_AutoQuestComplete"] = L["Quest completed2"],
      						["Sound\\Creature\\Peon\\PeonBuildingComplete1.wav"] = L["Work complete!"],
      					},
       					set = function(info, value) self.profile.questCompleteSound= value; qmstr.QuestMaster:PlayQuestCompleteSound(); self:Update() end,
       					get = function() return self.profile.questCompleteSound end,
      				}
      			}
			},
        	appearance = {
        		type = "group",
        		name = L["Appearance"],
        		order = _NextOrder(),
      			args = {
      				headerHeader = {
      					type = "header",
      					order = _NextOrder(),
      					name = L["Quest and achievement headers"],
      				},
    				headerColor = {
    					type = "color",
    					name = L["Header color"],
    					desc = L["Color used for the quest/achievement headers."],
    					order = _NextOrder(),
    					set = function(info, ...) self.profile.headerColor = {...}; self:Update() end,
    					get = function() return unpack(self.profile.headerColor) end,
    				},
    				nl1 = {
    					type = "description",
    					name = "",
    					order = _NextOrder()	
    				},
    				headerPaddingTop = {
    					type = "range",
    					name = L["Header padding top"],
    					desc = L["Space in pixel over a quest/achievement header."],
    					order = _NextOrder(),
    					min = 0,
    					max = 20,
    					set = function(info, value) self.profile.headerPaddingTop = value; self:Update() end,
    					get = function() return self.profile.headerPaddingTop end,
    				},
    				headerPaddingLeft = {
    					type = "range",
    					name = L["Header padding left"],
    					desc = L["Space in pixel before a quest/achievement header."],
    					order = _NextOrder(),
    					min = 0,
    					max = 100,
    					set = function(info, value) self.profile.headerPaddingLeft = value; self:Update() end,
    					get = function() return self.profile.headerPaddingLeft end,
    				},
    				nl2 = {
    					type = "description",
    					name = "\n\n",
    					order = _NextOrder()	
    				},
      				entryHeader = {
      					type = "header",
      					order = _NextOrder(),
      					name = L["Quest and achievement names"],
      				},
    				entryColor = {
    					type = "color",
    					name = L["Name color"],
    					desc = L["Color used for the quest/achievement names."],
    					order = _NextOrder(),
    					set = function(info, ...) self.profile.entryColor = {...}; self:Update() end,
    					get = function() return unpack(self.profile.entryColor) end,
    				},
    				colorizeQuestDifficulty = {
    					type = "toggle",
    					name = L["Colorize quests"],
    					desc = L["Colorize the names of the quests according to their difficulty."],
    					order = _NextOrder(),
    					set = function(info, value) self.profile.colorizeQuestDifficulty = value; self:Update() end,
    					get = function() return self.profile.colorizeQuestDifficulty end,
    				},
    				nl3 = {
    					type = "description",
    					name = "",
    					order = _NextOrder()	
    				},
    				entryPaddingTop = {
    					type = "range",
    					name = L["Name padding top"],
    					desc = L["Space in pixel over a quest/achievement name."],
    					order = _NextOrder(),
    					min = 0,
    					max = 20,
    					set = function(info, value) self.profile.entryPaddingTop = value; self:Update() end,
    					get = function() return self.profile.entryPaddingTop end,
    				},
    				entryPaddingLeft = {
    					type = "range",
    					name = L["Name padding left"],
    					desc = L["Space in pixel before a quest/achievement name."],
    					order = _NextOrder(),
    					min = 0,
    					max = 100,
    					set = function(info, value) self.profile.entryPaddingLeft = value; self:Update() end,
    					get = function() return self.profile.entryPaddingLeft end,
    				},
    				nl4 = {
    					type = "description",
    					name = "\n\n",
    					order = _NextOrder()	
    				},
      				objectiveHeader = {
      					type = "header",
      					order = _NextOrder(),
      					name = L["Quest and achievement objectives"],
      				},
    				objectiveColor = {
    					type = "color",
    					name = L["Objective color"],
    					desc = L["Color used for the quest/achievement objectives."],
    					order = _NextOrder(),
    					set = function(info, ...) self.profile.objectiveColor = {...}; self:Update() end,
    					get = function() return unpack(self.profile.objectiveColor) end,
    				},
    				colorizeQuestObjectives = {
    					type = "toggle",
    					name = L["Colorize objectives"],
    					desc = L["Colorize the objectives of quests or achievements according to their progress."],
    					order = _NextOrder(),
    					set = function(info, value) self.profile.colorizeQuestObjectives = value; self:Update() end,
    					get = function() return self.profile.colorizeQuestObjectives end,
    				},
    				nl5 = {
    					type = "description",
    					name = "",
    					order = _NextOrder()	
    				},
    				maxAchievementTextLength = {
    					type = "range",
    					name = L["Max achievement length"],
    					desc = L["Maximal length of the text for displaying all criterias of an achievement."],
    					order = _NextOrder(),
    					min = 50,
    					max = 1000,
    					step = 10,
    					bigStep = 50,
    					isPercent = false,
    					set = function(info, value) self.profile.maxAchievementTextLength = value; self:Update() end,
    					get = function() return self.profile.maxAchievementTextLength end,
    				},
    				maxAchievementCriteriaLength = {
    					type = "range",
    					name = L["Max criteria length"],
    					desc = L["Maximal length of a single criteria of an achievemenet."],
    					order = _NextOrder(),
    					min = 10,
    					max = 200,
    					step = 1,
    					bigStep = 5,
    					isPercent = false,
    					set = function(info, value) self.profile.maxAchievementCriteriaLength = value; self:Update() end,
    					get = function() return self.profile.maxAchievementCriteriaLength end,
    				},
    				nl6 = {
    					type = "description",
    					name = "",
    					order = _NextOrder()	
    				},
    				objectivePaddingTop = {
    					type = "range",
    					name = L["Objective padding top"],
    					desc = L["Space in pixel over a quest/achievement objective."],
    					order = _NextOrder(),
    					min = 0,
    					max = 20,
    					set = function(info, value) self.profile.objectivePaddingTop = value; self:Update() end,
    					get = function() return self.profile.objectivePaddingTop end,
    				},
    				objectivePaddingLeft = {
    					type = "range",
    					name = L["Objective padding left"],
    					desc = L["Space in pixel before a quest/achievement objective."],
    					order = _NextOrder(),
    					min = 0,
    					max = 100,
    					set = function(info, value) self.profile.objectivePaddingLeft = value; self:Update() end,
    					get = function() return self.profile.objectivePaddingLeft end,
    				},
        		}
        	},
        	profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(qmstr.QuestMaster.db) 
      	}
    }
end

local DEFAULT_PROFILE = {
	iconBars = {}
}

function qmstr.Config:OnInitialize()
	self.db = qmstr.QuestMaster.db
	self.profile = self.db.profile
	
	_InitOpts(self)
	
	AceConfig:RegisterOptionsTable("QuestMaster", self.opts)
	AceConfigDialog:SetDefaultSize("QuestMaster", 600, 580)
	
	self.db.RegisterCallback(self, "OnProfileChanged", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Reset")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
	
	qmstr.QuestMaster:RegisterChatCommand("questmaster", function() self:ToogleConfigDialog() end)
end

function qmstr.Config:Update()
	qmstr.QuestMaster:UpdateWindow(true)
end

function qmstr.Config:Reset()
	qmstr.QuestMaster:Reset()
end

function qmstr.Config:ToogleConfigDialog()
	log:Debug("Toggle")
	if (AceConfigDialog.OpenFrames["QuestMaster"]) then
		AceConfigDialog:Close("QuestMaster")
	else
		AceConfigDialog:Open("QuestMaster")
	end
end